@@ -65,6 +65,8 @@ typedef struct {
 #define PyGParamSpec_Check(v) (PyObject_TypeCheck(v, &PyGParamSpec_Type))
 
 typedef int (*PyGClassInitFunc) (gpointer gclass, PyTypeObject *pyclass);
+typedef PyTypeObject * (*PyGTypeRegistrationFunction) (const gchar *name,
+						       gpointer data);
 
 struct _PyGObject_Functions {
     /* 
@@ -143,14 +145,14 @@ struct _PyGObject_Functions {
 					const GParamSpec* pspec);
     PyTypeObject *enum_type;
     PyObject *(*enum_add)(PyObject *module,
-			  const char *typename_,
+			  const char *type_name_,
 			  const char *strip_prefix,
 			  GType gtype);
     PyObject* (*enum_from_gtype)(GType gtype, int value);
     
     PyTypeObject *flags_type;
     PyObject *(*flags_add)(PyObject *module,
-			   const char *typename_,
+			   const char *type_name_,
 			   const char *strip_prefix,
 			   GType gtype);
     PyObject* (*flags_from_gtype)(GType gtype, int value);
@@ -176,6 +178,10 @@ struct _PyGObject_Functions {
     void      (*add_warning_redirection) (const char *domain,
                                           PyObject   *warning);
     void      (*disable_warning_redirections) (void);
+    void      (*type_register_custom)(const gchar *type_name,
+				      PyGTypeRegistrationFunction callback,
+				      gpointer data);
+
 };
 
 #ifndef _INSIDE_PYGOBJECT_
@@ -221,8 +227,8 @@ struct _PyGObject_Functions *_PyGObject_API;
 #define pyg_param_spec_from_object  (_PyGObject_API->paramspec_get)
 #define pyg_pyobj_to_unichar_conv   (_PyGObject_API->pyobj_to_unichar_conv)
 #define pyg_parse_constructor_args  (_PyGObject_API->parse_constructor_args)
-#define pyg_param_gvalue_from_pyobject (_PyGObject_API->value_from_pyobject)
 #define pyg_param_gvalue_as_pyobject   (_PyGObject_API->value_as_pyobject)
+#define pyg_param_gvalue_from_pyobject (_PyGObject_API->gvalue_from_param_pyobject)
 #define PyGEnum_Type                (*_PyGObject_API->enum_type)
 #define pyg_enum_add                (_PyGObject_API->enum_add)
 #define pyg_enum_from_gtype         (_PyGObject_API->enum_from_gtype)
@@ -237,7 +243,7 @@ struct _PyGObject_Functions *_PyGObject_API;
 #define pyg_set_object_has_new_constructor (_PyGObject_API->set_object_has_new_constructor)
 #define pyg_add_warning_redirection   (_PyGObject_API->add_warning_redirection)
 #define pyg_disable_warning_redirections (_PyGObject_API->disable_warning_redirections)
-
+#define pyg_type_register_custom_callback (_PyGObject_API->type_register_custom)
 
 #define pyg_block_threads()   G_STMT_START {   \
     if (_PyGObject_API->block_threads != NULL) \
@@ -287,6 +293,33 @@ struct _PyGObject_Functions *_PyGObject_API;
     } \
 } G_STMT_END
 
+#define init_pygobject_check(req_major, req_minor, req_micro) G_STMT_START { \
+    init_pygobject();                                                        \
+    PyObject *gobject = PyImport_ImportModule("gobject");                    \
+    PyObject *mdict = PyModule_GetDict(gobject);                             \
+    PyObject *version = PyDict_GetItemString(mdict, "pygobject_version");    \
+    int found_major, found_minor, found_micro;                               \
+    if (!version)                                                            \
+        version = PyDict_GetItemString(mdict, "pygtk_version");              \
+    if (!version) {                                                          \
+        PyErr_SetString(PyExc_ImportError,                                   \
+                         "PyGObject version too old");                       \
+        return;                                                              \
+    }                                                                        \
+    if (!PyArg_ParseTuple(version, "iii",                                    \
+                          &found_major, &found_minor, &found_micro))         \
+        return;                                                              \
+    if (req_major != found_major ||                                          \
+        req_minor >  found_minor ||                                          \
+        req_minor == found_minor && req_micro > found_micro) {		     \
+        PyErr_Format(PyExc_ImportError,                                      \
+                     "PyGObject version mismatch, %d.%d.%d is required, "    \
+                     "found %d.%d.%d.", req_major, req_minor, req_micro,     \
+                     found_major, found_minor, found_micro);                 \
+        return;                                                              \
+    }                                                                        \
+} G_STMT_END
+
 #endif /* !_INSIDE_PYGOBJECT_ */
 
 G_END_DECLS
