@@ -168,5 +168,78 @@ class TestEmissionHook(unittest.TestCase):
             self.assertEqual(e.status, 1)
         e.status = 3
 
+class TestClosures(unittest.TestCase):
+    def setUp(self):
+        self.count = 0
+
+    def _callback(self, e):
+        self.count += 1
+
+    def testDisconnect(self):
+        e = E()
+        e.connect('signal', self._callback)
+        e.disconnect_by_func(self._callback)
+        e.emit('signal')
+        self.assertEqual(self.count, 0)
+
+    def testHandlerBlock(self):
+        e = E()
+        e.connect('signal', self._callback)
+        e.handler_block_by_func(self._callback)
+        e.emit('signal')
+        self.assertEqual(self.count, 0)
+
+    def testHandlerUnBlock(self):
+        e = E()
+        signal_id = e.connect('signal', self._callback)
+        e.handler_block(signal_id)
+        e.handler_unblock_by_func(self._callback)
+        e.emit('signal')
+        self.assertEqual(self.count, 1)
+
+    def testGString(self):
+        class C(gobject.GObject):
+            __gsignals__ = { 'my_signal': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_GSTRING,
+                                           (gobject.TYPE_GSTRING,)) }
+            def __init__(self, test):
+                gobject.GObject.__init__(self)
+                self.test = test
+            def do_my_signal(self, data):
+                self.data = data
+                self.test.assertEqual(len(data), 3)
+                return ''.join([data[2], data[1], data[0]])
+        c = C(self)
+        data = c.emit("my_signal", "\01\00\02")
+        self.assertEqual(data, "\02\00\01")
+
+class SigPropClass(gobject.GObject):
+    __gsignals__ = { 'my_signal': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
+                                   (gobject.TYPE_INT,)) }
+
+    __gproperties__ = {
+        'foo': (str, None, None, '', gobject.PARAM_WRITABLE|gobject.PARAM_CONSTRUCT),
+        }
+
+    signal_emission_failed = False
+
+    def do_my_signal(self, arg):
+        self.arg = arg
+
+    def do_set_property(self, pspec, value):
+        if pspec.name == 'foo':
+            self._foo = value
+        else:
+            raise AttributeError, 'unknown property %s' % pspec.name
+        try:
+            self.emit("my-signal", 1)
+        except TypeError:
+            self.signal_emission_failed = True
+
+
+class TestSigProp(unittest.TestCase):
+    def testEmitInPropertySetter(self):
+        obj = SigPropClass()
+        self.failIf(obj.signal_emission_failed)
+
 if __name__ == '__main__':
     unittest.main()
